@@ -1,7 +1,7 @@
 from .utils import get_class_names
 from .voc import VOC
 from .fat import FAT
-# from .ycb_video import YCBVideo
 from .open_images import OpenImages
 from .ferplus import FERPlus
 from .fer import FER
+from .cityscapes import CityScapes
