@@ -60,7 +60,8 @@ class AugmentDetection(SequentialProcessor):
             for encoding bounding boxes.
     """
     def __init__(self, prior_boxes, split=pr.TRAIN, num_classes=21, size=300,
-                 mean=pr.BGR_IMAGENET_MEAN, IOU=.5, variances=[.1, .2]):
+                 mean=pr.BGR_IMAGENET_MEAN, IOU=.5,
+                 variances=[0.1, 0.1, 0.2, 0.2]):
         super(AugmentDetection, self).__init__()
         # image processors
         self.augment_image = AugmentImage()
@@ -94,13 +95,17 @@ class DetectSingleShot(Processor):
         score_thresh: Float between [0, 1]
         nms_thresh: Float between [0, 1].
         mean: List of three elements indicating the per channel mean.
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
     """
     def __init__(self, model, class_names, score_thresh, nms_thresh,
-                 mean=pr.BGR_IMAGENET_MEAN):
+                 mean=pr.BGR_IMAGENET_MEAN, variances=[0.1, 0.1, 0.2, 0.2],
+                 draw=True):
         self.model = model
         self.class_names = class_names
         self.score_thresh = score_thresh
         self.nms_thresh = nms_thresh
+        self.variances = variances
+        self.draw = draw
 
         super(DetectSingleShot, self).__init__()
         preprocessing = SequentialProcessor(
@@ -111,19 +116,20 @@ class DetectSingleShot(Processor):
              pr.ExpandDims(axis=0)])
         postprocessing = SequentialProcessor(
             [pr.Squeeze(axis=None),
-             pr.DecodeBoxes(self.model.prior_boxes, variances=[.1, .2]),
+             pr.DecodeBoxes(self.model.prior_boxes, self.variances),
              pr.NonMaximumSuppressionPerClass(self.nms_thresh),
              pr.FilterBoxes(self.class_names, self.score_thresh)])
         self.predict = pr.Predict(self.model, preprocessing, postprocessing)
 
         self.denormalize = pr.DenormalizeBoxes2D()
-        self.draw = pr.DrawBoxes2D(self.class_names)
+        self.draw_boxes2D = pr.DrawBoxes2D(self.class_names)
         self.wrap = pr.WrapOutput(['image', 'boxes2D'])
 
     def call(self, image):
         boxes2D = self.predict(image)
         boxes2D = self.denormalize(image, boxes2D)
-        image = self.draw(image, boxes2D)
+        if self.draw:
+            image = self.draw_boxes2D(image, boxes2D)
         return self.wrap(image, boxes2D)
 
 
@@ -133,6 +139,7 @@ class SSD512COCO(DetectSingleShot):
     # Arguments
         score_thresh: Float between [0, 1]
         nms_thresh: Float between [0, 1].
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
 
     # Example
         ``` python
@@ -153,11 +160,11 @@ class SSD512COCO(DetectSingleShot):
         - [SSD: Single Shot MultiBox
             Detector](https://arxiv.org/abs/1512.02325)
     """
-    def __init__(self, score_thresh=0.60, nms_thresh=0.45):
+    def __init__(self, score_thresh=0.60, nms_thresh=0.45, draw=True):
         model = SSD512()
         names = get_class_names('COCO')
         super(SSD512COCO, self).__init__(
-            model, names, score_thresh, nms_thresh)
+            model, names, score_thresh, nms_thresh, draw=draw)
 
 
 class SSD512YCBVideo(DetectSingleShot):
@@ -166,6 +173,7 @@ class SSD512YCBVideo(DetectSingleShot):
     # Arguments
         score_thresh: Float between [0, 1]
         nms_thresh: Float between [0, 1].
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
 
     # Example
         ``` python
@@ -185,11 +193,11 @@ class SSD512YCBVideo(DetectSingleShot):
 
 
     """
-    def __init__(self, score_thresh=0.60, nms_thresh=0.45):
+    def __init__(self, score_thresh=0.60, nms_thresh=0.45, draw=True):
         names = get_class_names('YCBVideo')
         model = SSD512(weights='YCBVideo', num_classes=len(names))
         super(SSD512YCBVideo, self).__init__(
-            model, names, score_thresh, nms_thresh)
+            model, names, score_thresh, nms_thresh, draw=draw)
 
 
 class SSD300VOC(DetectSingleShot):
@@ -198,6 +206,7 @@ class SSD300VOC(DetectSingleShot):
     # Arguments
         score_thresh: Float between [0, 1]
         nms_thresh: Float between [0, 1].
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
 
     # Example
         ``` python
@@ -219,10 +228,11 @@ class SSD300VOC(DetectSingleShot):
         - [SSD: Single Shot MultiBox
             Detector](https://arxiv.org/abs/1512.02325)
     """
-    def __init__(self, score_thresh=0.60, nms_thresh=0.45):
+    def __init__(self, score_thresh=0.60, nms_thresh=0.45, draw=True):
         model = SSD300()
         names = get_class_names('VOC')
-        super(SSD300VOC, self).__init__(model, names, score_thresh, nms_thresh)
+        super(SSD300VOC, self).__init__(
+            model, names, score_thresh, nms_thresh, draw=draw)
 
 
 class SSD300FAT(DetectSingleShot):
@@ -231,6 +241,7 @@ class SSD300FAT(DetectSingleShot):
     # Arguments
         score_thresh: Float between [0, 1]
         nms_thresh: Float between [0, 1].
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
 
     # Example
         ``` python
@@ -248,10 +259,11 @@ class SSD300FAT(DetectSingleShot):
         inferences and a list of ``paz.abstract.messages.Boxes2D``.
 
     """
-    def __init__(self, score_thresh=0.60, nms_thresh=0.45):
+    def __init__(self, score_thresh=0.60, nms_thresh=0.45, draw=True):
         model = SSD300(22, 'FAT', 'FAT')
         names = get_class_names('FAT')
-        super(SSD300FAT, self).__init__(model, names, score_thresh, nms_thresh)
+        super(SSD300FAT, self).__init__(
+            model, names, score_thresh, nms_thresh, draw=draw)
 
 
 class DetectHaarCascade(Processor):
@@ -261,8 +273,7 @@ class DetectHaarCascade(Processor):
         detector: An instantiated ``HaarCascadeDetector`` model.
         offsets: List of two elements. Each element must be between [0, 1].
         class_names: List of strings.
-        draw: Boolean flag. If ``True`` the prediction will be drawn
-            in the image.
+        draw: Boolean. If ``True`` prediction are drawn in the returned image.
 
     # Returns
         A function for predicting bounding box detections.
